package com.mt.wms.empty.controller;

import com.mt.wms.basic.service.KilnInfoService;
import com.mt.wms.basic.vo.KilnInfoVo;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.CurrTask;
import com.mt.wms.core.params.BasePageParam;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.ApmsCreateProcessSheet;
import com.mt.wms.empty.params.CreateItem;
import com.mt.wms.empty.params.CurrTaskQueryParam;
import com.mt.wms.empty.params.TaskCreateParam;
import com.mt.wms.empty.service.CurrTaskDetService;
import com.mt.wms.empty.service.CurrTaskService;
import com.mt.wms.empty.service.OrderInfoService;
import com.mt.wms.empty.vo.ApmsCreateProcessSheetVo;
import com.mt.wms.empty.vo.ApmsStoveVo;
import com.mt.wms.empty.vo.CurrTaskMainQueryVo;
import com.mt.wms.empty.vo.CurrTaskQueryVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Map;

/**
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "currTask")
@Slf4j
@Api(value = "任务管理", tags = "当前任务管理", hidden = false)
public class CurrTaskController extends BaseController {
    @Autowired
    private CurrTaskService currTaskService;
    @Autowired
    private CurrTaskDetService currTaskDetService;
    @Autowired
    private OrderInfoService orderInfoService;
    @Autowired
    private ApmsController apmsControl;
    @Autowired
    private KilnInfoService kilnInfoService;

    @PostMapping(value = "createProcessTask")
    @ApiOperation(value = "订单加工-创建一个加工任务至任务队列")
    public R<String> createProcessTask(@Validated @RequestBody TaskCreateParam param) {
        //验证标识卡号正确无误。
        orderInfoService.verifyTaskInfoByIdenCardNum(param.getDetParams());
        //验证炉子编码信息在apms正确无误
        R<KilnInfoVo> kilnInfoVoR = kilnInfoService.get(IdParam.builder().id(param.getKilnId()).build());
        String kilnCode = kilnInfoVoR.getData().getCode();
        R<ApmsStoveVo> bm = apmsControl.getStoveCodeByWorkShopCode("BM");
        ArrayList<Map<String, String>> stoveCodes = bm.getData().getStoveCodes();
        //验证apms存在这个炉子
        boolean verifyCodes = false;
        ArrayList<String> codeStr = new ArrayList<>();
        stoveCodes.forEach(kilnMap -> {
            codeStr.addAll(kilnMap.keySet());
        });
        for (String s : codeStr)
        {
            if (s.equals(kilnCode))
            {
                verifyCodes = true;
                break;
            }
        }
        if (!verifyCodes)
        {
            return failed(kilnCode + " 验证APMS炉号失败！");
        }

        //通知apms创建一个生产单
        ApmsCreateProcessSheet apmsCreateSheet = new ApmsCreateProcessSheet();
        apmsCreateSheet.setWorkShopCode("BM");
        apmsCreateSheet.setStartTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        apmsCreateSheet.setTechCode(param.getCraftCode());
        //fixme 暂时没有user信息
        apmsCreateSheet.setStartUser("");
        ArrayList<CreateItem> createItems = new ArrayList<>();
        param.getDetParams().forEach(e -> {
            CreateItem createItem = new CreateItem();
            createItem.setItemCode(e.getIdenCardNum());
            createItem.setQuantity(e.getQuantity().intValue());
            createItem.setWeight(e.getWeight().doubleValue());
            createItems.add(createItem);
        });
        apmsCreateSheet.setItems(createItems);
        R<ApmsCreateProcessSheetVo> processSheet = apmsControl.createProcessSheet(apmsCreateSheet);

        if (processSheet.getData().getSuccess())
        {
            //apms创建成功后写入数据库
            R<IdVo> mainTask = currTaskService.createProcessTask(param);
            currTaskDetService.createProcessTaskDet(param.getDetParams(), mainTask.getData().getId());
            String sheetNo = processSheet.getData().getSheetNo();
            //将返回的生产单号写入到主任务中。
            CurrTask currTaskById = currTaskService.getCurrTaskById(mainTask.getData().getId());
            currTaskById.setSheetNo(sheetNo);
            currTaskService.updateCurrTaskById(currTaskById);
            return successful("创建成功，任务添加至等待执行队列。");
        }
        else
        {
            return failed("创建失败，APMS报错：" + processSheet.getMsg());
        }
    }


    @PostMapping(value = "mainCurrentTaskNow")
    @ApiOperation(value = "首页-获取当前执行中的任务")
    public R<PageVo<CurrTaskMainQueryVo>> mainCurrentTaskNow(@Validated @RequestBody BasePageParam param) {
        return currTaskService.currentTaskMainPage(param);
    }

    @PostMapping(value = "currentTaskNow")
    @ApiOperation(value = "获取当前执行的任务 (车辆名称传id)")
    public R<PageVo<CurrTaskQueryVo>> currentTask(@Validated @RequestBody CurrTaskQueryParam param) {
        return currTaskService.currentTaskPage(param);
    }

    @PostMapping(value = "completeTask")
    @ApiOperation(value = "执行任务完成（仅wms，不通知apms）")
    public R<String> completeTask(@Validated @RequestBody IdParam param) {
        return currTaskService.completeTask(param);
    }

    @PostMapping(value = "deleteBySheetNo")
    @ApiOperation(value = "人工通过生产单编号删除未正式开始的生产单(通知APMS)")
    public R<String> deleteBySheetNo(@Validated @RequestBody String sheetNo) {
        return currTaskService.deleteBySheetNo(sheetNo);
    }
}
